

/*//////////////////////////////////////////////////////////////////
[ RESTYLE TAG ]*/
* {
	margin: 0px; 
	padding: 0px; 
	box-sizing: border-box;
}

body, html {
	height: 100%;
	font-family: sans-serif;
}

/* ------------------------------------ */
a {
	margin: 0px;
	transition: all 0.4s;
	-webkit-transition: all 0.4s;
  -o-transition: all 0.4s;
  -moz-transition: all 0.4s;
}

a:focus {
	outline: none !important;
}

a:hover {
	text-decoration: none;
}

/* ------------------------------------ */
h1,h2,h3,h4,h5,h6 {margin: 0px;}

p {margin: 0px;}

ul, li {
	margin: 0px;
	list-style-type: none;
}


/*//////////////////////////////////////////////////////////////////
[ Table ]*/

.limiter {
  width: 100%;
  margin: 0 auto;
  font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
}

.container-table100 {
  width: 100%;
  min-height: 100vh;
  background: #c4d3f6;

  display: -webkit-box;
  display: -webkit-flex;
  display: -moz-box;
  display: -ms-flexbox;
  display: flex;
  align-items: center;
  justify-content: center;
  flex-wrap: wrap;
  padding: 33px 30px;
}

.wrap-table100 {
  width: 960px;
  border-radius: 10px;
  overflow: hidden;
}

table {
  width: 100%;
  display: table;
  margin: 0;
}

td, th {
  text-align: center;
  padding: 10px;
}

@media screen and (max-width: 768px) {
  table {
    display: block;
  }
}

tr {
  display: table-row;
  background: #fff;
}

thead tr {
  color: #ffffff;
  background: #6c7ae0;
}

@media screen and (max-width: 768px) {
  tr {
    display: block;
  }

  thead tr {
    padding: 0;
    height: 0px;
  }

  thead tr th {
    display: none;
  }

  tr td:before {
    font-size: 12px;
    color: #808080;
    line-height: 1.2;
    text-transform: uppercase;
    font-weight: unset !important;

    margin-bottom: 13px;
    content: attr(data-title);
    min-width: 98px;
    display: block;
  }
}

td {
  display: table-cell;
}

@media screen and (max-width: 768px) {
  td {
    display: block;
  }
}

tr td {
  font-size: 15px;
  color: #666666;
  line-height: 1.2;
  font-weight: unset !important;

  padding-top: 20px;
  padding-bottom: 20px;
  border-bottom: 1px solid #f2f2f2;
}

tbody tr th {
  font-size: 18px;
  color: #fff;
  line-height: 1.2;
  font-weight: unset !important;

  padding-top: 19px;
  padding-bottom: 19px;
}

table, tr {
  width: 100% !important;
}

tr:hover {
  background-color: #ececff;
  cursor: pointer;
}

@media (max-width: 768px) {
  tr {
    border-bottom: 1px solid #f2f2f2;
    padding-bottom: 18px;
    padding-top: 30px;
    padding-right: 15px;
    margin: 0;
  }
  
  tr td {
    border: none;
    padding-left: 30px;
    padding-top: 16px;
    padding-bottom: 16px;
  }
  tr td:nth-child(1) {
    padding-left: 30px;
  }
  
  tr td {
    font-size: 18px;
    color: #555555;
    line-height: 1.2;
    font-weight: unset !important;
  }

  table, tr, td {
    width: 100% !important;
  }
}

.green-color {
  color: green;
}

.red-color {
  color: darkred;
}

.orange-color {
  color: darkorange;
}