<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
    <title>wily report</title>
    <link rel="stylesheet" href="css/main.css" type="text/css">
</head>
<body class="indexfile">
    <div class="limiter">
		<div class="container-table100">
			<div class="wrap-table100">
                <table class="index">
                    <thead>
                        <tr>$headers</tr>
                    </thead>
                    <tbody>
                        $content
                    </tbody>
                </table>
            </div>
        </div>
    </div>
</body>
</html>