#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['wily', 'wily.archivers', 'wily.commands', 'wily.helper', 'wily.operators']

package_data = \
{'': ['*'], 'wily': ['templates/*', 'templates/css/*']}

install_requires = \
['gitpython', 'radon', 'click', 'colorlog', 'tabulate', 'plotly', 'progress']

extras_require = \
{":python_version == '3.6'": ['dataclasses']}

entry_points = \
{'console_scripts': ['wily = wily.__main__:cli']}

setup(name='wily',
      version='1.12.1',
      description='Wily.',
      author='Anthony Shaw',
      author_email='anthonyshaw@apache.org',
      url='https://github.com/tonybaloney/wily',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
